@@ -11,7 +11,7 @@ our @EXPORT_OK   = qw();
 
 use Data::Dumper;
 use Text::ParseWords qw(shellwords);
-use List::Util qw(first);
+use List::Util qw(first pairmap);
 
 use Getopt::EX::Func qw(parse_func);
 
@@ -20,6 +20,7 @@ sub new {
     my $obj = bless {
 	Module => undef,
 	Base => undef,
+	Mode => { FUNCTION => 0, WILDCARD => 0 },
 	Define => [],
 	Expand  => [],
 	Option => [],
@@ -139,6 +140,15 @@ sub expand {
     s{ (\$ENV\{ (['"]?) \w+ \g{-1} \}) }{ eval($1) // $1 }xge;
 }
 
+sub mode {
+    my $obj = shift;
+    @_ == 1 and return $obj->{Mode}->{uc shift};
+    die "Unexpected parameter." if @_ % 2;
+    pairmap {
+	$obj->{Mode}->{uc $a} = $b;
+    } @_;
+}
+
 use constant BUILTIN => "__BUILTIN__";
 sub validopt { $_[0] ne BUILTIN }
 
@@ -213,6 +223,38 @@ sub getlocal {
     @e;
 }
 
+sub expand_args {
+    my $obj = shift;
+    my @args = @_;
+
+    ##
+    ## Expand `&function' style arguments.
+    ##
+    if ($obj->mode('function')) {
+	@args = map {
+	    if (/^&(.+)/) {
+		my $func = parse_func $obj->module . "::$1";
+		$func ? $func->call : $_;
+	    } else {
+		$_;
+	    }
+	}
+	@args;
+    }
+
+    ##
+    ## Expand wildcards.
+    ##
+    if ($obj->mode('wildcard')) {
+	@args = map {
+	    my @glob = glob $_;
+	    @glob ? @glob : $_;
+	} @args;
+    }
+
+    @args;
+}
+
 sub default {
     my $obj = shift;
     $obj->getopt('default', DEFAULT => 1);
@@ -271,7 +313,9 @@ sub parseline {
     my $line = shift;
     my @arg = split ' ', $line, 3;
 
-    if (@arg < 3) {
+    my %min_args = ( mode => 1, DEFAULT => 3 );
+    my $min_args = $min_args{$arg[0]} || $min_args{DEFAULT};
+    if (@arg < $min_args) {
 	warn sprintf("Parse error in %s: %s\n", $obj->title, $line);
 	return;
     }
@@ -322,6 +366,14 @@ sub parseline {
 	shift @arg;
 	$obj->autoload(@arg);
     }
+    elsif ($arg[0] eq "mode") {
+	shift @arg;
+	for (@arg) {
+	    if (/^(no-?)?(.*)/i) {
+		$obj->mode($2 => $1 ? 0 : 1);
+	    }
+	}
+    }
     elsif ($arg[0] eq "help") {
 	$obj->help($arg[1], $arg[2]);
     }
@@ -599,4 +651,23 @@ Get built-in options.
 
 Set autoload module.
 
+=item B<mode>
+
+Set argument treatment mode.  Arguments produced by option expansion
+will be the subject of post-process.  This method define the behavior
+of it.
+
+=over 4
+
+=item B<mode>(B<function> => 1)
+
+Interpret the argument start with '&' as a function, and replace it by
+the result of the function call.
+
+=item B<mode>(B<wildcard> => 1)
+
+Replace wildcard argument by matched file names.
+
+=back
+
 =back
